#!/usr/bin/python
from __future__ import division
from scipy import optimize
import numpy as np

def fitSplines(x,y,degree=3,smfactor=None,fitranges=None,normedsmooth=True):
    """
    x and y are the data to be fit (as 1-D arrays)
    degree is the degree (order-1) of the splines
    smfactor is the smoothing factor given to UnivariateSpline
    if normedsmooth is true, the smoothing factor is normalized by the square of the mean
    fitranges is a sequence of 2-tuples with indicies into the ranges to
    be use for fitting - e.g. [(2,8),(12,18),(45,60)]

    return value is fit,dev,bounds,splines where fit is the function, dev
    is the rms deviation, bounds are is a list of2-tuples(lowerbound,upperboud)
    matching the splines in the splines list. Note that all the splines overlap
    at the endpoints,so the top ends (except for the last one) should be excluded
    """
    from scipy.interpolate import UnivariateSpline

    allx=x
    ally=y
    if fitranges is not None:
        x,y=_extractRanges(x,y,fitranges)

    if normedsmooth:
        smfactor=smfactor*y.var()

    spline=UnivariateSpline(x,y,k=degree,s=smfactor)

    fit = spline(allx)
    dev=np.std(ally-fit)

    coeffs=spline.get_coeffs()
    knots=spline.get_knots()

    return fit,dev,{'coeffs':coeffs,'knots':knots,'n':len(knots),'period':(max(allx)-min(allx))/len(knots)},spline

def fitPowerLaw(x,y,A=None,alpha=None,fitranges=None,method='lsq',tol=1E-8):
    fullx,fully=x,y
    if fitranges is not None:
        x,y=_extractRanges(x,y,fitranges)
    
    #power law of the form y = A * x^alpha
    def f(x,A,alpha):
        return A*x**alpha
    
    fp=FuncWithPars(f,{'A':1 if A is  None else A,'alpha':1 if alpha is None else alpha})

    res=fp.fitPars(x,y,tol=tol,method=method)
    dev=fp.computeFitRMSDev(fullx,fully)

    return fp(fullx),dev,fp.getPars(),res[1:]

def fit2PowerModel(x,y,A=1,xc=1,alpha=1,beta=3,fixedpars=('alpha','beta'),fitranges=None,method='lsq',tol=1E-8,integratedy=False):
    """
    generic functions of the form A/(x^alpha*(x+xc)^(beta-alpha))
    sometimes these are written as B/((x/xc)^alpha*(1+x/xc)^(beta-alpha) where B = A/xc^beta
    """
    
    fullx,fully=x,y
    if fitranges is not None:
        x,y=_extractRanges(x,y,fitranges)
        
    def f(x,A,xc,alpha,beta):
        return A*x**(-alpha)*(x+xc)**(alpha-beta)    
    if integratedy:
        from math import pi
        from scipy.integrate import quad
        def fint(x,A,xc,alpha,beta):
            vquad=np.vectorize(lambda x:quad(f,0,x,args=(4*pi*A,xc,alpha-2,beta-2),epsabs=tol)[0]) #alpha-2 is for thing with stuff
            return vquad(x)
        fp=FuncWithPars(fint,{'A':A,'xc':xc,'alpha':alpha,'beta':beta})
    else:
        fp=FuncWithPars(f,{'A':A,'xc':xc,'alpha':alpha,'beta':beta})
    
    
    
    for p in fixedpars:
        fp.fixPar(p)

    res=fp.fitPars(x,y,tol=tol,method=method)
    dev=fp.computeFitRMSDev(fullx,fully)

    return fp(fullx),dev,fp.getPars(),res[1:]

def fitPolynomial(x,y,degree,icoeffs=None,fitranges=None,tol=1E-8):
    """
    returns yfit, rms_deviation,coefficients,extras (extras are the result of optimize.leastsq)
    """
    #raise NotImplementedError
    from scipy.optimize import leastsq
    from numpy import mean,linspace,std,polyval

    if icoeffs is None:
        icoeffs=[]
        for n in range(degree+1):
            icoeffs.append(mean(y)/(degree+1))
    else:
        if len(icoeffs) != degree+1:
            raise ValueError('wrong number of initial coefficients for requested degree')
    
    allx=x
    ally=y

    if fitranges is not None:
        x,y=_extractRanges(x,y,fitranges)
        
    def f(p,x,y):
        return y-polyval(p,x)

    #TODO:test if y,x, and polys should by passed in implicitly instead of through args
    res=leastsq(f,icoeffs,ftol=tol,args=(x,y))
    bestcoeffs=res[0]

    fit = polyval(bestcoeffs,allx)
    return fit,std(ally-fit),{'coeffs':bestcoeffs,'degree':degree},res[1:]

def fitChebyshevPolynomial(x,y,degree,chebyshevtype='t',icoeffs=None,fitranges=None,tol=1E-8):
    """
    returns yfit, rms_deviation,coefficients,extras (extras are the result of optimize.leastsq)

    chebyshevtype can be 'u','t','s', or 'c'
    """
    from scipy.special import chebyu,chebyt,chebys,chebyc
    from scipy.optimize import leastsq
    from numpy import mean,linspace,std

    if x is None:
        x=linspace(-1,1,len(y))
    else:
        #compress x down to [-1,1], the orthogonal range for Legendres
        x=2*x/(max(x)-min(x))
        x-=min(x)+1

    allx=x
    ally=y

    if fitranges is not None:
        x,y=_extractRanges(x,y,fitranges)

    if icoeffs is None:
        icoeffs=[]
        for n in range(degree+1):
            icoeffs.append(mean(y)/(degree+1))
    else:
        if len(icoeffs) != degree+1:
            raise ValueError('wrong number of initial coefficients for requested degree')

    if chebyshevtype=='t':
        polys=[chebyt(n) for n in range(degree+1)]
    elif chebyshevtype=='u':
        polys=[chebyu(n) for n in range(degree+1)]
    elif chebyshevtype=='s':
        polys=[chebys(n) for n in range(degree+1)]
    elif chebyshevtype=='c':
        polys=[chebyc(n) for n in range(degree+1)]
    else:
        raise ValueError("unknown code for chebyshev polynomial type")


    def f(coeffs,y,x,polys):
        result=0
        for i,p in enumerate(polys):
            result+=coeffs[i]*p(x)
        return result-y

    #TODO:test if y,x, and polys should by passed in implicitly instead of through args
    res=leastsq(f,icoeffs,ftol=tol,args=(y,x,polys))
    bestcoeffs=res[0]

    fit = ally+f(bestcoeffs,ally,allx,polys)
    return fit,std(ally-fit),{'coeffs':bestcoeffs,'degree':degree},res[1:]



def fitLegendrePolynomial(x,y,degree,icoeffs=None,fitranges=None,tol=1E-8):
    """
    returns yfit, rms_deviation,coefficients,extras (extras are the result of optimize.leastsq)
    """
    from scipy.special import legendre
    from scipy.optimize import leastsq
    from numpy import mean,linspace,std

    if icoeffs is not None and len(icoeffs)!=order+1:
        raise ValueError('number of initial coefficients does not match order+1')
    if x is None:
        x=linspace(-1,1,len(y))
    else:
        #compress x down to [-1,1], the orthogonal range for Legendres
        x=2*x/(max(x)-min(x))
        x-=min(x)+1

    allx=x
    ally=y

    if fitranges is not None:
        x,y=_extractRanges(x,y,fitranges)

    if icoeffs is None:
        icoeffs=[]
        mean=mean(y)
        for n in range(degree+1):
            icoeffs.append(mean/(degree+1))
    else:
        if len(icoeffs) != order+1:
            raise ValueError('wrong number of initial coefficients for requested order')

    polys=[legendre(n) for n in range(degree+1)]


    def f(coeffs,y,x,polys):
        result=0
        for i,p in enumerate(polys):
            result+=coeffs[i]*p(x)
        return result-y

    #TODO:test if y,x, and polys should by passed in implicitly instead of through args
    res=leastsq(f,icoeffs,ftol=tol,args=(y,x,polys))
    bestcoeffs=res[0]

    fit = ally+f(bestcoeffs,ally,allx,polys)
    return fit,std(ally-fit),{'degree':degree,'coeffs':bestcoeffs},res[1:]

def planckPeakToA(peakval,T):
    h=6.626068E-11 #angstroms^2 g/s=6.626068E-27 cm^2 g/s = erg*s
    c=2.99792458E18 #angstroms/sec=3E10 cm/sec
    k=1.3806503 #angstrom^2 g/s^2/K  1.3806503E-16 cm^2 g/s^2/K=erg/K
    
    alpha=4
    wlmax=h*c/k/alpha
    
def planckBB(x,A,T,wl=True):
    h=6.626068E-11 #angstroms^2 g/s=6.626068E-27 cm^2 g/s = erg*s
    c=2.99792458E18 #angstroms/sec=3E10 cm/sec
    k=1.3806503 #angstrom^2 g/s^2/K  1.3806503E-16 cm^2 g/s^2/K=erg/K
    from numpy import exp
    
    if wl:
        return A*2*h*c*c*x**-5/(exp((h*c/(k*T*x)))-1)
    else:
        return A*2*h/c/c*x**3/(exp(h*x/(k*T))-1)

def fitPlanckBB(x,y,T0=5800,A0=None,wl=True,method='lsq',fitranges=None,tol=1E-8):
    """
    This fits a Plankian blackbody spectrum:
    f_lambda=A*2*h*c^2*lambda^-5/(E^(h c/lambda kT)-1)
    or
    f_nu=A*2*h*c^-2*nu^3/(E^(h nu/kT)-1)

    with nu in Hz, lambda in angstroms, and T in K
    if x is None, a linear x-axis will be assumed (starting at 1)

    tol is either ftol or (ftol,xtol)

    the return value is (yfit,rms_deviation,(A,T),extras), where extras depend on the
    fitting method used (just it's return value without the leading element)
    """
    from numpy import exp

    h=6.626068E-11 #angstroms^2 g/s=6.626068E-27 cm^2 g/s = erg*s
    c=2.99792458E18 #angstroms/sec=3E10 cm/sec
    k=1.3806503 #angstrom^2 g/s^2/K  1.3806503E-16 cm^2 g/s^2/K=erg/K

    fullx=x
    fully=y
    if fitranges is not None:
        x,y=_extractRanges(x,y,fitranges)

    if x is None:
        x=np.arange(len(y))+1

    if wl:
        def f(x,A,T):
            return A*2*h*c*c*x**-5/(exp((h*c/(k*T*x)))-1)
    else:
        def f(x,A,T):
            return A*2*h/c/c*x**3/(exp(h*x/(k*T))-1)
    if A0 is None:
        A0=y.mean()*(x.mean()**5 if wl else x.mean()**-3)/(2*h*c*c)
    fp=FuncWithPars(f,{'A':A0,'T':T0})

    res=fp.fitPars(x,y,tol=tol,method=method)
    dev=fp.computeFitRMSDev(fullx,fully)

    return fp(fullx),dev,fp.getPars(),res[1:]

def fitGaussian(x,y,A0=1,sigma0=1,mu0=0,C0=0,method='lsq',fitranges=None,tol=1E-8,fixedpars=('mu','C')):
    from math import pi
    from numpy import exp
    fullx=x
    fully=y
    if fitranges is not None:
        x,y=_extractRanges(x,y,fitranges)

    if x is None:
        x=np.arange(len(y))+1

    sqrttwopi=(2*pi)**0.5
    def f(x,A,sigma,mu,C):
        return C+A*exp(-(x-mu)**2*sigma**-2/2)*sigma**-0.5/sqrttwopi
    fp=FuncWithPars(f,{'A':A0,'sigma':sigma0,'mu':mu0,'C':C0})

    for p in fixedpars:
        fp.fixPar(p)

    res=fp.fitPars(x,y,tol=tol,method=method)
    dev=fp.computeFitRMSDev(fullx,fully)

    return fp(fullx),dev,fp.getPars(),res[1:]

def fitDoubleGaussian(x,y,A0=1,B0=-1,sigmaA0=1,sigmaB0=1,muA0=0,muB0=0,C0=0,method='lsq',fitranges=None,tol=1E-8,fixedpars=('muA','muB','C')):
    """
    this will fit two gaussians simultaneously, assuming one to be negative and the other positive
    """
    from math import pi
    from numpy import exp

    fullx=x
    fully=y
    if fitranges is not None:
        x,y=_extractRanges(x,y,fitranges)

    if x is None:
        x=np.arange(len(y))+1

    sqrttwopi=(2*pi)**0.5
    #more readable version
    #def f(x,A,sigmaA,B,sigmaB,muA,muB,C):
    #    pA=abs(A)*exp(-(x-muA)**2*sigmaA**-2/2)*(sigmaA)**-0.5
    #    pB=abs(B)*exp(-(x-muB)**2*sigmaB**-2/2)*(sigmaB)**-0.5
    #    return C+(pA-pB)/sqrttwopi
    def f(x,A,sigmaA,B,sigmaB,muA,muB,C):
        return C+(abs(A)*exp(-(x-muA)**2*sigmaA**-2/2)*(sigmaA)**-0.5-abs(B)*exp(-(x-muB)**2*sigmaB**-2/2)*(sigmaB)**-0.5)/sqrttwopi
    fp=FuncWithPars(f,{'A':A0,'B':B0,'sigmaA':sigmaA0,'sigmaB':sigmaB0,'muA':muA0,'muB':muB0,'C':C0})

    for p in fixedpars:
        fp.fixPar(p)

    res=fp.fitPars(x,y,tol=tol,method=method)
    dev=fp.computeFitRMSDev(fullx,fully)

    return fp(fullx),dev,fp.getPars(),res[1:]

def fitLorentzian(x,y,A0=1,xs0=1,mu0=0,C0=0,method='lsq',fitranges=None,tol=1E-8,fixedpars=('mu','C')):
    from numpy import exp

    fullx=x
    fully=y
    if fitranges is not None:
        x,y=_extractRanges(x,y,fitranges)

    if x is None:
        x=np.arange(len(y))+1

    def f(x,A,xs,mu,C):
        return C+A/(((x-mu)/xs)**2+1)
    fp=FuncWithPars(f,{'A':A0,'xs':xs0,'mu':mu0,'C':C0})

    for p in fixedpars:
        fp.fixPar(p)

    res=fp.fitPars(x,y,tol=tol,method=method)
    dev=fp.computeFitRMSDev(fullx,fully)

    return fp(fullx),dev,fp.getPars(),res[1:]

def fitVoigt(x,y,A0=1,G0=1,L0=1,sigma0=1,xs0=1,mu0=0,C0=0,method='lsq',fitranges=None,tol=1E-8,fixedpars=('mu','C','A')):
    from math import pi
    from numpy import exp,convolve
    
    print 'voigt fit faulty?',A0,G0,L0
    print 'fixed',fixedpars


    fullx=xr
    fully=y
    if fitranges is not None:
        x,y=_extractRanges(x,y,fitranges)

    if x is None:
        x=np.arange(len(y))+1

    sqrttwopi=(2*pi)**0.5
    def f(x,A,G,L,sigma,xs,mu,C):
        gaussian=G*exp(-(x-mu)**2*sigma**-2/2)*sigma**-0.5/sqrttwopi
        lorentzian=L/(((x-mu)/xs)**2+1)/pi/xs
        return C+A*convolve(gaussian,lorentzian,mode='same')
    fp=FuncWithPars(f,{'A':A0,'sigma':sigma0,'xs':xs0,'mu':mu0,'C':C0,'G':G0,'L':L0})

    for p in fixedpars:
        fp.fixPar(p)

    res=fp.fitPars(x,y,tol=tol,method=method)
    dev=fp.computeFitRMSDev(fullx,fully)

    return fp(fullx),dev,fp.getPars(),res[1:]

def _extractRanges(x,y,ranges):
    x=np.concatenate([x[l:(u+1)] for l,u in ranges])
    y=np.concatenate([y[l:(u+1)] for l,u in ranges])
    return x,y

class FuncWithPars(object):
    """
    the main purpose of this class is to abstract out the ordering of the various
    scipy optimization algorithms and allow just about any kind of input scheme for
    functions that want some of the parameters to be optimized.  This of course means
    a performance hit, but most of the hard work is usually done in the fitting itself,
    so for many applications this doesn't really matter
    """
    #TODO: ~85% as fast as lsqfit - maybe improvable?
    def __init__(self,func,params0):
        """
        func is the function to use that takes N_P+1 arguments (parameter names must match
        initial parameters) - the data input will be whichever isn't a parameter
        parms0 is a mapping with keys as the parameters and valus the initial
        values for those parameters

        alternatively, func may take 1 regular argument and a vararg (not a keyword vararg), in which
        case the keys to params0 should be integers from 0 to N, and the function will be called with
        the vararg as a list of length N

        as a third option, func can take 1 regular argument and a varkwarg (not a *arg), in which case
        params0 will be passed in as the kwargs dict
        """
        from inspect import getargspec
        # equivalent to self.__params=params0, but needed for __getattr__ overriding
        object.__setattr__(self, '_FuncWithPars__params', params0)
        self.__func=func
        self.__fixedparams=set()
        self.__jacobian=None
        self.__fprime=None


        argspec=getargspec(func)
        if argspec[1] is not None:
            if argspec[2] is not None:
                raise ValueError('supplied function cannot have both varargs and kwargs')
            #vararg mode
        elif argspec[2] is not None:
            #kwarg style - the parameters are set directly from
            self.__call__=self.__callkwargs
        elif len(argspec[0])== len(params0)+1:
            args=list(argspec[0])

            parnames=params0.keys()
            self.__dataind=None
            self.__parinds={}
            for i,a in enumerate(argspec[0]):
                if a in parnames:
                    self.__parinds[a]=i
                else:
                    if self.__dataind is None:
                        self.__dataind=i
                    else:
                        raise ValueError("param0 keys don't match function argument names")

            if self.__dataind is None:
                raise ValueError("some very bizare interaction is occuring between param0's keys and the function's argument list")
            #TODO:test that this works
            self.__call__=self.__callregargs
        else:
            raise ValueError("supplied function does not have the right number of arguments (should by len(params0)+1")


    def setJacobian(self,jfunc):
        """
        jacobian can be a function to take the same input as func and compute the
        Jacobian of func with derivatives across the rows
        """
        self.__jacobian=jfunc

    def setGradient(self,fpfunc):
        """
        a function to compute the gradient
        """
        self.__fprime=fpfunc

    def __call__(self,x):
        return self.__call__(x)

    def __callregargs(self,x):
        args=range(len(self.__parinds)+1)
        for k,i in self.__parinds.iteritems():
            args[i]=self.__params[k]
        args[self.__dataind]=x
        return self.__func(*args)

    def __callvarargs(self,x):
        #not sure if its guaranteed for these keys to be sorted - watch out if they aren't!
        params=self.__params
        args=[params[k] for k in params]
        return self.__func(x,*args)

    def __callkwargs(self,x):
        return self.__func(x,**self.__params)

    def setPars(self,d):
        raise NotImplementedError('get from other comp')

    def getPars(self):
        return self.__params.copy()

    def fixPar(self,param):
        if param not in self.__params.keys():
            raise KeyError("Couldn't find the requested parameter "+param)

        self.__fixedparams.add(param)

    def freePar(self,param):
        if param not in self.__params.keys():
            raise KeyError("Couldn't find the requested parameter "+param)

        if param in self.__fixedparams:
            self.__fixedparams.remove(param)

    def fitPars(self,x,y,method='lsq',tol=1E-8,**otherkwargs):
        """
        this will optimize the parameters to achieve a best-fit using a variety of fitting options

        otherkwargs depends on the method used

        tol should be either ftol or gtol (depending on if ftol is an option), or (ftol,xtol) or (ftol,xtol,gtol)

        return values are whatever the scipy.optimize function for the specified method returns
        """
        import numpy as n
        import scipy.optimize as opt
        if x==None:
            x=np.arange(len(y))

        if np.isscalar(tol):
            tol=(tol,1e-8,1e-8)
        elif len(tol)==1:
            tol=tol=(tol[0],1e-8,1e-8)
        elif len(tol)==2:
            tol=(tol[0],tol[1],1e-8)
        elif len(tol)==3:
            tol=(tol[0],tol[1],tol[2])
        else:
            tol=(1e-8,1e-8,1e-8)


        p0=[self.__params[k] for k in self.__params.keys() if k not in self.__fixedparams]
        parkeys=[k for k in self.__params.keys() if k not in self.__fixedparams]
        #TODO:compare performance difference of having y explicit here versus args=(y) in the calling optimizers
        def f(parin):
            for i,k in enumerate(parkeys):
                self.__params[k]=parin[i]

            return y - self(x)

        def fsum(parin):
            for i,k in enumerate(parkeys):
                self.__params[k]=parin[i]

            return np.sum((y - self(x))**2)



        if method == 'lsq':
            res=opt.leastsq(f, p0, Dfun=self.__jacobian, full_output=True,ftol=tol[0],xtol=tol[1],**otherkwargs)
        elif method == 'fmin':
            res=opt.fmin(fsum, p0, full_output=True,ftol=tol[0],**otherkwargs)
        elif method=='fmin_powell':
            res=opt.fmin_powell(fsum, p0, full_output=True,ftol=tol[0],**otherkwargs)
        elif method=='fmin_cg':
            if self.__fprime is None:
                raise Exception('fmin_cg requires a gradient')
            res=opt.fmin_cg(f, p0,self.__fprime,full_output=True,gtol=tol[0],**otherkwargs)
        elif method=='fmin_bfgs':
            if self.__fprime is None:
                raise Exception('fmin_bfgs requires a gradient')
            res=opt.fmin_bfgs(f, p0,self.__fprime, full_output=True,maxiters=maxiters,ftol=tol[0],xtol=tol[1],gtol=tol[2],**otherkwargs)
        else:
            raise ValueError('method is not a recognized optimization techniuqe')

        return res

    def computeFitRMSDev(self,x,y):
        import numpy as n
        res=self(x)
        return np.std(y-res)
    
    def plotRange(self,xstart,xend,num=200,**kwargs):
        from pylab import plot
        x=np.linspace(xstart,xend,num)
        plot(x,self(x),**kwargs)

    def __getattr__(self,name):
        if name in self.__params:
            return self.__params[name]
        else:
            raise AttributeError

    def __setattr__(self, name, value):
        if name in self.__params:
            self.__params[name]=value
        else:
            object.__setattr__(self, name, value)
    def __getitem__(self,key):
        return self.__params[key]

    def __setitem__(self,key,val):
        self.__params[key]=val

    def __iter__(self):
        """
        gives an iteritems of the parameter dictionary
        """
        return self.__params.iteritems()


#this hiding technique is faster but somewhat dirty and less class-oriented
#adapted from http://new.scipy.org/Cookbook/FittingData
class Parameter:
        def __init__(self, value):
                self.value = value

        def set(self, value):
                self.value = value

        def __call__(self):
                return self.value

def lsqfit(function, parameters, y, x = None,lskwargs={}):
    """
    function is a python function that is simply the fit function (NOT residuals) - the function should accept
    a single input (the x-alue) and return a single value with parameters as function calls (e.g. lamda x: m()*x+B() )
    parameters is a list of Parameter objects with names matching the function usage
    y and x are the data
    """
    def f(params):
        for i,p in enumerate(parameters):
            p.set(params[i])
        return y - function(x)

    if x is None: x = np.arange(y.shape[0])
    p = [param() for param in parameters]
    return optimize.leastsq(f, p,**lskwargs)

def linear_lsq(x,y,fixint=False,fixslope=False,sd=False):
    """
    does least-squares fit on the x,y data
    
    fixint and fixslope can be used to specify the intercept or slope of the fit
    or leave them free by having fixint or fixslope be False or None
    
    return m,b,dy,dm,db or m,b,dy,dm,db,std
    """
    if fixint is None:
        fixint=False
    if fixslope is None:
        fixslope=False
    
    N=len(x) 
    if fixint is False and fixslope is False:
        if len(y)!=N:
            raise ValueError('data arrays are not same length!')
        sxsq=np.sum(x*x)
        sx,sy=np.sum(x),np.sum(y)
        sxy=np.sum(x*y)
        delta=N*sxsq-sx**2
        m=(N*sxy-sx*sy)/delta
        b=(sxsq*sy-sx*sxy)/delta
        dy=(y-m*x-b).std(ddof=2)
        dm=dy*(sxsq/delta)**0.5
        db=dy*(N/delta)**0.5
        
    elif fixslope is not False:
        m,dm=fixslope,0
        sxsq=np.sum(x*x)
        b=np.sum(y-m*x)/N
        
        dy=(y-m*x-b).std(ddof=1)
        db=dy #TODO:check
        
    elif fixint is not False:
        b,db=fixslope,0
        sxy=np.sum(x*y)
        sxsq=np.sum(x*x)
        m=(sxy-N*b)/sxsq
        
        dy=(y-m*x-b).std(ddof=1)
        dm=dy*(sxsq/N)**0.5
    else:
        raise ValueError("can't fix both slope and intercept")
    
    if sd:
        fit=x*m+b
        return m,b,dy,dm,db,np.std(y-fit)
    else:
        return m,b,dy,dm,db

def weighted_linear_lsq(x,y,sigmay=None,doplot=False):
    """
    does a linear weighted least squares fit and computes the coefficients and errors
    fit is y=B*x+A
    if sigma is None, the weights are all equal - otherwise, it's the stddev of the y values
    returns B,A,sigmaB,sigmaA
    """
    from numpy import array,ones,sum
    if sigmay is None:
        sigmay=ones(len(x))
    else:
        sigmay=array(sigmay)
    if len(x)!=len(y)!=len(sigmay):
        raise ValueError('arrays not matching lengths')
    N=len(x)
    x,y=array(x),array(y)
    
    w=1.0/sigmay/sigmay
    delta=sum(w)*sum(w*x*x)-(sum(w*x))**2
    A=(sum(w*x*x)*sum(w*y)-sum(w*x)*sum(w*x*y))/delta
    B=(sum(w)*sum(w*x*y)-sum(w*x)*sum(w*y))/delta
    diff=y-A-B*x
    sigmaysq=sum(w*diff*diff)/(sum(w)*(N-2)/N) #TODO:check
    sigmaA=(sigmaysq*sum(w*x*x)/delta)**0.5
    sigmaB=(sigmaysq*sum(w)/delta)**0.5
    
    if doplot:
        from matplotlib.pyplot import plot,errorbar,legend
        errorbar(x,y,sigmay,fmt='o',label='Data')
        plot(x,B*x+A,label='Best Fit')
        plot(x,(B+sigmaB)*x+A-sigmaA,label='$1\sigma$ Up')
        plot(x,(B-sigmaB)*x+A+sigmaA,label='$1\sigma$ Down')
        legend(loc=0)
    
    return B,A,sigmaB,sigmaA

